<?php
/*
 +-------------------------------------------------------------------------+
 | Copyright (C) 2004 Ian Berry                                            |
 |                                                                         |
 | This program is free software; you can redistribute it and/or           |
 | modify it under the terms of the GNU General Public License             |
 | as published by the Free Software Foundation; either version 2          |
 | of the License, or (at your option) any later version.                  |
 |                                                                         |
 | This program is distributed in the hope that it will be useful,         |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
 | GNU General Public License for more details.                            |
 +-------------------------------------------------------------------------+
 | cacti: a php-based graphing solution                                    |
 +-------------------------------------------------------------------------+
 | Most of this code has been designed, written and is maintained by       |
 | Ian Berry. See about.php for specific developer credit. Any questions   |
 | or comments regarding this code should be directed to:                  |
 | - iberry@raxnet.net                                                     |
 +-------------------------------------------------------------------------+
 | - raXnet - http://www.raxnet.net/                                       |
 +-------------------------------------------------------------------------+
*/

chdir('../../');


include("./include/auth.php");
include_once("./lib/utility.php");

define("MAX_DISPLAY_PAGES", 21);
include_once("./include/top_header.php");
poller_perf();
include_once("./include/bottom_footer.php");

/* --------------------------
    Global Form Functions
   -------------------------- */

function poller_perf() {
	global $colors;

	/* ================= input validation ================= */
	input_validate_input_number(get_request_var_request("page"));
	/* ==================================================== */
	$logfile 				= read_config_option("path_cactilog");
	$i 						= 0;
	$poller_runs 			= 0;	# number of poller runs, count "SYSTEM STATS"
	$stats_found			= 0;	# number of poller runs, that include runtime statistics
	/* search string			for Host entries */
	$search_Host 			= '/^.+\s+(\d*:\d*:\d* [A|P]M).+Host\[(\d*)\](.*)/';
	/* search string			for DS entries */
	$search_DS 				= '/^.+DS\[\d*\]/';
	/* search string			for Errors or Warnings */
	$search_ErrorWarning 	= '/^.*ERROR|WARNING/';
	/* search string			for SYSTEM STATS */
	$search_System_Stats 	= '/^.*SYSTEM STATS/';
	/* remember these fields in session vars so we don't have to keep passing them around */
	load_current_session_value("page", "sess_device_current_page", "1");
	
	html_start_box("<strong>Poller Performance Overview</strong>", "98%", $colors["header"], "3", "center", "");

	$total_rows = db_fetch_cell("select
		COUNT(host.id)
		from host");

	/* get $hosts() from MySql */
	$hosts = db_fetch_assoc("select
		host.id,
		host.disabled,
		host.status,
		host.hostname,
		host.description
		from host
		order by host.description
		limit " . (read_config_option("num_rows_device")*($_REQUEST["page"]-1)) . "," . read_config_option("num_rows_device"));


	/* open log/cacti.log for reading, hope we get it while cacti is NOT writing to it */
	/* to get $loghosts() array                                                        */
    $fp = fopen($logfile, "r");
    
    while ($line = fgets($fp, 4096)) {
    	/* 
    	 * search for Host entries
    	 * 03/04/2006 09:55:03 PM - CACTID: Poller[0] Host[1] SNMP Result: Host responded to SNMP
    	 * and remember time, host_id and trailing text
    	 */

    	
    	/* filter WARNING and ERROR messages, they may contain Host[..] entries so better ignore them */
        if ( preg_match($search_ErrorWarning, $line, $match) ) {continue; }

        
        /* get Host entries */
        if ( preg_match($search_Host, $line, $match) ) {
        	$time 		= $match[1];
        	$host_id 	= $match[2];
        	$trail 		= $match[3];

			/* store all log data in loghosts array */
        	if (!isset($loghosts[$host_id]["start"])) {
        		/* this is the first entry for this host_id? */
        		$loghosts[$host_id]["start"] = $time;
        		$loghosts[$host_id]["ds_count"] = 0;
        	}
        	/* remember each entry for a host as its last one */
        	$loghosts[$host_id]["end"] = $time;
			/* count if DS entry found */
	        if ( preg_match($search_DS, $trail, $match) ) { $loghosts[$host_id]["ds_count"]++; }
        }

    	
    	/* if SYSTEM STATS, where at the end of a poller's run */
        if ( preg_match($search_System_Stats, $line, $match) ) {
        	$poller_runs++;
			if (isset($loghosts)) {$stats_found++;}
        	
			/* merge $hosts() and $loghosts() arrays here */
			for ($i = 0; $i < sizeof($hosts); $i++) {
				/* found sth for this host in the logfile? $hosts[$i]["id"] is id of current host*/
				$current_id = $hosts[$i]["id"];
				$found = isset($loghosts[$current_id]["end"]);
				if ($found) {
					$hosts[$i]["lstart"] 	= $loghosts[$current_id]["start"];
					$hosts[$i]["lend"] 		= $loghosts[$current_id]["end"];
					if (isset($hosts[$i]["ds_count"])) {
						$hosts[$i]["ds_count"] += $loghosts[$current_id]["ds_count"];
					} else {
						$hosts[$i]["ds_count"] = $loghosts[$current_id]["ds_count"];
					}
					if (isset($hosts[$i]["diff"])) {
						/* add 0.5 sec due to time resolution */
						$hosts[$i]["diff"]	+= strtotime($loghosts[$current_id]["end"]) -  strtotime($loghosts[$current_id]["start"]) + 0.5;
					} else {
						$hosts[$i]["diff"]	= strtotime($loghosts[$current_id]["end"]) -  strtotime($loghosts[$current_id]["start"]) + 0.5;
					}
					if ($hosts[$i]["ds_count"] > 0) {
						$hosts[$i]["rate"]	= $hosts[$i]["diff"]/$hosts[$i]["ds_count"];
					} else {
						$hosts[$i]["rate"]	= 0;
					}
				}
			}
			/* destroy loghosts array to start from scratch for new poller logrecords */
			unset($loghosts);
		}

    }
    fclose($fp);


	/* generate page list */
	print "<tr><td>Poller Runs including Statistics: " . $stats_found . "</td><td></td><td>" . "Total Runs: " . $poller_runs . "</td></tr>";
	$url_page_select = get_page_list($_REQUEST["page"], MAX_DISPLAY_PAGES, read_config_option("num_rows_device"), $total_rows, "poller-perf.php?");

	$nav = "<tr bgcolor='#" . $colors["header"] . "'>
			<td colspan='11'>
				<table width='100%' cellspacing='0' cellpadding='0' border='0'>
					<tr>
						<td align='left' class='textHeaderDark'>
							<strong>&lt;&lt; "; if ($_REQUEST["page"] > 1) { $nav .= "<a class='linkOverDark' href='poller-perf.php?page=" . ($_REQUEST["page"]-1) . "'>"; } $nav .= "Previous"; if ($_REQUEST["page"] > 1) { $nav .= "</a>"; } $nav .= "</strong>
						</td>\n
						<td align='center' class='textHeaderDark'>
							Showing Rows " . ((read_config_option("num_rows_device")*($_REQUEST["page"]-1))+1) . " to " . ((($total_rows < read_config_option("num_rows_device")) || ($total_rows < (read_config_option("num_rows_device")*$_REQUEST["page"]))) ? $total_rows : (read_config_option("num_rows_device")*$_REQUEST["page"])) . " of $total_rows [$url_page_select]
						</td>\n
						<td align='right' class='textHeaderDark'>
							<strong>"; if (($_REQUEST["page"] * read_config_option("num_rows_device")) < $total_rows) { $nav .= "<a class='linkOverDark' href='poller-perf.php?page=" . ($_REQUEST["page"]+1) . "'>"; } $nav .= "Next"; if (($_REQUEST["page"] * read_config_option("num_rows_device")) < $total_rows) { $nav .= "</a>"; } $nav .= " &gt;&gt;</strong>
						</td>\n
					</tr>
				</table>
			</td>
		</tr>\n";

	print $nav;

	html_header(array("Description", "Status", "Hostname", "Last<br>Poller Start", "Last<br>Poller End", "#DS<br>Total", "Duration<br>Total", "#DS<br>per run", "Duration<br>per run", "Duration<br>per DS"));

	$i = 0;
	if (sizeof($hosts) > 0) {
		foreach ($hosts as $host) {
			form_alternate_row_color($colors["alternate"],$colors["light"],$i); $i++;
				?>
				<td width=250>
					<a class="linkEditMain" href="host.php?action=edit&id=<?php print $host["id"];?>"><?php print $host["description"];?></a>
				</td>
				<td><?php print get_colored_device_status(($host["disabled"] == "on" ? true : false), $host["status"]);?></td>
				<td><?php print $host["hostname"];?></td>
				<td><?php print (isset($host["lstart"]) 	? $host["lstart"] 							: "None");?></td>
				<td><?php print (isset($host["lend"]) 		? $host["lend"] 							: "None");?></td>
				<td><?php print (isset($host["ds_count"]) 	? $host["ds_count"] 						: "None");?></td>
				<td><?php print (isset($host["diff"]) 		? $host["diff"] 							: "None");?></td>
				<td><?php print (isset($host["ds_count"]) 	? round($host["ds_count"]/$stats_found, 2) 	: "None");?></td>
				<td><?php print (isset($host["diff"]) 		? round($host["diff"]/$stats_found, 2) 		: "None");?></td>
				<td><?php print (isset($host["rate"]) 		? round($host["rate"], 2) 					: "None");?></td>
			</tr>
			<?php
		}

		/* put the nav bar on the bottom as well */
		print $nav;
	}else{
		print "<tr><td><em>No Hosts</em></td></tr>";
	}
	html_end_box(false);
}

?>
